@@ -1,6 +1,6 @@
 /*  Plzip - A parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010, 2011, 2012 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -63,7 +63,7 @@ namespace {
 
 const char * const Program_name = "Plzip";
 const char * const program_name = "plzip";
-const char * const program_year = "2010";
+const char * const program_year = "2012";
 const char * invocation_name = 0;
 
 #ifdef O_BINARY
@@ -97,50 +97,55 @@ void show_help() throw()
   {
   std::printf( "%s - A parallel compressor compatible with lzip.\n", Program_name );
   std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
-  std::printf( "\nOptions:\n" );
-  std::printf( "  -h, --help                 display this help and exit\n" );
-  std::printf( "  -V, --version              output version information and exit\n" );
-  std::printf( "  -B, --data-size=<n>        set input data block size in bytes\n" );
-  std::printf( "  -c, --stdout               send output to standard output\n" );
-  std::printf( "  -d, --decompress           decompress\n" );
-  std::printf( "  -f, --force                overwrite existing output files\n" );
-  std::printf( "  -k, --keep                 keep (don't delete) input files\n" );
-  std::printf( "  -m, --match-length=<n>     set match length limit in bytes [36]\n" );
-  std::printf( "  -n, --threads=<n>          set the number of (de)compression threads\n" );
-  std::printf( "  -o, --output=<file>        if reading stdin, place the output into <file>\n" );
-  std::printf( "  -q, --quiet                suppress all messages\n" );
-  std::printf( "  -s, --dictionary-size=<n>  set dictionary size limit in bytes [8MiB]\n" );
-  std::printf( "  -t, --test                 test compressed file integrity\n" );
-  std::printf( "  -v, --verbose              be verbose (a 2nd -v gives more)\n" );
-  std::printf( "  -1 .. -9                   set compression level [default 6]\n" );
-  std::printf( "      --fast                 alias for -1\n" );
-  std::printf( "      --best                 alias for -9\n" );
+  std::printf( "\nOptions:\n"
+               "  -h, --help                     display this help and exit\n"
+               "  -V, --version                  output version information and exit\n"
+               "  -B, --data-size=<bytes>        set input data block size in bytes\n"
+               "  -c, --stdout                   send output to standard output\n"
+               "  -d, --decompress               decompress\n"
+               "  -f, --force                    overwrite existing output files\n"
+               "  -F, --recompress               force recompression of compressed files\n"
+               "  -k, --keep                     keep (don't delete) input files\n"
+               "  -m, --match-length=<bytes>     set match length limit in bytes [36]\n"
+               "  -n, --threads=<n>              set the number of (de)compression threads\n"
+               "  -o, --output=<file>            if reading stdin, place the output into <file>\n"
+               "  -q, --quiet                    suppress all messages\n"
+               "  -s, --dictionary-size=<bytes>  set dictionary size limit in bytes [8MiB]\n"
+               "  -t, --test                     test compressed file integrity\n"
+               "  -v, --verbose                  be verbose (a 2nd -v gives more)\n"
+               "  -1 .. -9                       set compression level [default 6]\n"
+               "      --fast                     alias for -1\n"
+               "      --best                     alias for -9\n" );
   if( verbosity > 0 )
     {
     std::printf( "  -D, --debug=<level>        (0-1) print debug statistics to stderr\n" );
     }
-  std::printf( "If no file names are given, %s compresses or decompresses\n", program_name );
-  std::printf( "from standard input to standard output.\n" );
-  std::printf( "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n" );
-  std::printf( "Ki = KiB = 2^10 = 1024, M = 10^6, Mi = 2^20, G = 10^9, Gi = 2^30, etc...\n" );
-  std::printf( "\nReport bugs to lzip-bug@nongnu.org\n" );
-  std::printf( "Plzip home page: http://www.nongnu.org/lzip/plzip.html\n" );
+  std::printf( "If no file names are given, plzip compresses or decompresses\n"
+               "from standard input to standard output.\n"
+               "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n"
+               "Ki = KiB = 2^10 = 1024, M = 10^6, Mi = 2^20, G = 10^9, Gi = 2^30, etc...\n"
+               "The bidimensional parameter space of LZMA can't be mapped to a linear\n"
+               "scale optimal for all files. If your files are large, very repetitive,\n"
+               "etc, you may need to use the --match-length and --dictionary-size\n"
+               "options directly to achieve optimal performance.\n"
+               "\nReport bugs to lzip-bug@nongnu.org\n"
+               "Plzip home page: http://www.nongnu.org/lzip/plzip.html\n" );
   }
 
 
 void show_version() throw()
   {
   std::printf( "%s %s\n", Program_name, PROGVERSION );
-  std::printf( "Copyright (C) 2009 Laszlo Ersek.\n" );
-  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
+  std::printf( "Copyright (C) 2009 Laszlo Ersek.\n"
+               "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
   std::printf( "Using Lzlib %s\n", LZ_version() );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
-  std::printf( "This is free software: you are free to change and redistribute it.\n" );
-  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
+  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
+               "This is free software: you are free to change and redistribute it.\n"
+               "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
-long long getnum( const char * const ptr, const int bs = 0,
+long long getnum( const char * const ptr,
                   const long long llimit = LLONG_MIN + 1,
                   const long long ulimit = LLONG_MAX ) throw()
   {
@@ -161,9 +166,6 @@ long long getnum( const char * const ptr, const int bs = 0,
     switch( tail[0] )
       {
       case ' ': break;
-      case 'b': if( bs > 0 ) { factor = bs; exponent = 1; }
-                else bad_multiplier = true;
-                break;
       case 'Y': exponent = 8; break;
       case 'Z': exponent = 7; break;
       case 'E': exponent = 6; break;
@@ -205,7 +207,7 @@ int get_dict_size( const char * const arg ) throw()
   if( bits >= LZ_min_dictionary_bits() &&
       bits <= LZ_max_dictionary_bits() && *tail == 0 )
     return ( 1 << bits );
-  return getnum( arg, 0, LZ_min_dictionary_size(), LZ_max_dictionary_size() );
+  return getnum( arg, LZ_min_dictionary_size(), LZ_max_dictionary_size() );
   }
 
 
@@ -224,13 +226,13 @@ int extension_index( const std::string & name ) throw()
 
 int open_instream( const std::string & name, struct stat * const in_statsp,
                    const Mode program_mode, const int eindex,
-                   const bool force, const bool to_stdout ) throw()
+                   const bool recompress, const bool to_stdout ) throw()
   {
   int infd = -1;
-  if( program_mode == m_compress && !force && eindex >= 0 )
+  if( program_mode == m_compress && !recompress && eindex >= 0 )
     {
     if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Input file `%s' already has `%s' suffix.\n",
+      std::fprintf( stderr, "%s: Input file '%s' already has '%s' suffix.\n",
                     program_name, name.c_str(),
                     known_extensions[eindex].from );
     }
@@ -240,7 +242,7 @@ int open_instream( const std::string & name, struct stat * const in_statsp,
     if( infd < 0 )
       {
       if( verbosity >= 0 )
-        std::fprintf( stderr, "%s: Can't open input file `%s': %s.\n",
+        std::fprintf( stderr, "%s: Can't open input file '%s': %s.\n",
                       program_name, name.c_str(), std::strerror( errno ) );
       }
     else
@@ -253,10 +255,10 @@ int open_instream( const std::string & name, struct stat * const in_statsp,
       if( i != 0 || ( !S_ISREG( mode ) && ( !to_stdout || !can_read ) ) )
         {
         if( verbosity >= 0 )
-          std::fprintf( stderr, "%s: Input file `%s' is not a regular file%s.\n",
+          std::fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
                         program_name, name.c_str(),
                         ( can_read && !to_stdout ) ?
-                        " and `--stdout' was not specified" : "" );
+                        " and '--stdout' was not specified" : "" );
         close( infd );
         infd = -1;
         }
@@ -286,8 +288,8 @@ void set_d_outname( const std::string & name, const int i ) throw()
       }
     }
   output_filename = name; output_filename += ".out";
-  if( verbosity >= 0 )
-    std::fprintf( stderr, "%s: Can't guess original name for `%s' -- using `%s'.\n",
+  if( verbosity >= 1 )
+    std::fprintf( stderr, "%s: Can't guess original name for '%s' -- using '%s'.\n",
                   program_name, name.c_str(), output_filename.c_str() );
   }
 
@@ -301,10 +303,10 @@ bool open_outstream( const bool force ) throw()
   if( outfd < 0 && verbosity >= 0 )
     {
     if( errno == EEXIST )
-      std::fprintf( stderr, "%s: Output file `%s' already exists, skipping.\n",
+      std::fprintf( stderr, "%s: Output file '%s' already exists, skipping.\n",
                     program_name, output_filename.c_str() );
     else
-      std::fprintf( stderr, "%s: Can't create output file `%s': %s.\n",
+      std::fprintf( stderr, "%s: Can't create output file '%s': %s.\n",
                     program_name, output_filename.c_str(), std::strerror( errno ) );
     }
   return ( outfd >= 0 );
@@ -334,10 +336,10 @@ void cleanup_and_fail( const int retval ) throw()
     {
     delete_output_on_interrupt = false;
     if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Deleting output file `%s', if it exists.\n",
-               program_name, output_filename.c_str() );
+      std::fprintf( stderr, "%s: Deleting output file '%s', if it exists.\n",
+                    program_name, output_filename.c_str() );
     if( outfd >= 0 ) { close( outfd ); outfd = -1; }
-    if( std::remove( output_filename.c_str() ) != 0 )
+    if( std::remove( output_filename.c_str() ) != 0 && errno != ENOENT )
       show_error( "WARNING: deletion of output file (apparently) failed." );
     }
   std::exit( retval );
@@ -347,30 +349,26 @@ void cleanup_and_fail( const int retval ) throw()
      // Set permissions, owner and times.
 void close_and_set_permissions( const struct stat * const in_statsp )
   {
-  bool error = false;
+  bool warning = false;
   if( in_statsp )
     {
-    if( fchmod( outfd, in_statsp->st_mode ) != 0 ||
-        ( fchown( outfd, in_statsp->st_uid, in_statsp->st_gid ) != 0 &&
-          errno != EPERM ) ) error = true;
     // fchown will in many cases return with EPERM, which can be safely ignored.
+    if( ( fchown( outfd, in_statsp->st_uid, in_statsp->st_gid ) != 0 &&
+          errno != EPERM ) ||
+        fchmod( outfd, in_statsp->st_mode ) != 0 ) warning = true;
     }
-  if( close( outfd ) == 0 ) outfd = -1;
-  else cleanup_and_fail( 1 );
+  if( close( outfd ) != 0 ) cleanup_and_fail( 1 );
+  outfd = -1;
   delete_output_on_interrupt = false;
-  if( !in_statsp ) return;
-  if( !error )
+  if( in_statsp )
     {
     struct utimbuf t;
     t.actime = in_statsp->st_atime;
     t.modtime = in_statsp->st_mtime;
-    if( utime( output_filename.c_str(), &t ) != 0 ) error = true;
+    if( utime( output_filename.c_str(), &t ) != 0 ) warning = true;
     }
-  if( error )
-    {
+  if( warning && verbosity >= 1 )
     show_error( "Can't change output file attributes." );
-    cleanup_and_fail( 1 );
-    }
   }
 
 
@@ -397,8 +395,9 @@ void set_signals() throw()
 int verbosity = 0;
 
 
-// This can be called from any thread, main thread or sub-threads alike, since
-// they all call common helper functions that call fatal() in case of an error.
+// This can be called from any thread, main thread or sub-threads alike,
+// since they all call common helper functions that call fatal() in case
+// of an error.
 //
 void fatal() { signal_handler( SIGUSR1 ); }
 
@@ -432,13 +431,13 @@ void show_error( const char * const msg, const int errcode, const bool help ) th
       std::fprintf( stderr, "\n" );
       }
     if( help && invocation_name && invocation_name[0] )
-      std::fprintf( stderr, "Try `%s --help' for more information.\n",
+      std::fprintf( stderr, "Try '%s --help' for more information.\n",
                     invocation_name );
     }
   }
 
 
-void internal_error( const char * const msg )
+void internal_error( const char * const msg ) throw()
   {
   if( verbosity >= 0 )
     std::fprintf( stderr, "%s: internal error: %s.\n", program_name, msg );
@@ -477,7 +476,7 @@ int writeblock( const int fd, const uint8_t * const buf, const int size ) throw(
     errno = 0;
     const int n = write( fd, buf + size - rest, rest );
     if( n > 0 ) rest -= n;
-    else if( errno && errno != EINTR && errno != EAGAIN ) break;
+    else if( n < 0 && errno != EINTR && errno != EAGAIN ) break;
     }
   return ( rest > 0 ) ? size - rest : size;
   }
@@ -507,6 +506,7 @@ int main( const int argc, const char * const argv[] )
   Mode program_mode = m_compress;
   bool force = false;
   bool keep_input_files = false;
+  bool recompress = false;
   bool to_stdout = false;
   std::string input_filename;
   std::string default_output_filename;
@@ -518,11 +518,8 @@ int main( const int argc, const char * const argv[] )
   if( LZ_version()[0] != LZ_version_string[0] )
     internal_error( "bad library version" );
 
-  const int slots_per_worker = 2;
   long max_workers = sysconf( _SC_THREAD_THREADS_MAX );
-  if( max_workers < 1 || max_workers > INT_MAX / slots_per_worker )
-    max_workers = INT_MAX / slots_per_worker;
-  if( max_workers > INT_MAX / (int)sizeof (pthread_t) )
+  if( max_workers < 1 || max_workers > INT_MAX / (int)sizeof (pthread_t) )
     max_workers = INT_MAX / sizeof (pthread_t);
 
   const Arg_parser::Option options[] =
@@ -541,9 +538,9 @@ int main( const int argc, const char * const argv[] )
     { 'B', "data-size",       Arg_parser::yes },
     { 'c', "stdout",          Arg_parser::no  },
     { 'd', "decompress",      Arg_parser::no  },
-    { 'e', "extreme",         Arg_parser::no  },
     { 'D', "debug",           Arg_parser::yes },
     { 'f', "force",           Arg_parser::no  },
+    { 'F', "recompress",      Arg_parser::no  },
     { 'h', "help",            Arg_parser::no  },
     { 'k', "keep",            Arg_parser::no  },
     { 'm', "match-length",    Arg_parser::yes },
@@ -573,20 +570,20 @@ int main( const int argc, const char * const argv[] )
       case '5': case '6': case '7': case '8': case '9':
                 encoder_options = option_mapping[code-'0']; break;
       case 'b': break;
-      case 'B': data_size = getnum( arg, 0, 2 * LZ_min_dictionary_size(),
+      case 'B': data_size = getnum( arg, 2 * LZ_min_dictionary_size(),
                                     2 * LZ_max_dictionary_size() ); break;
       case 'c': to_stdout = true; break;
       case 'd': program_mode = m_decompress; break;
-      case 'D': debug_level = getnum( arg, 0, 0, 3 ); break;
-      case 'e': break;				// ignored by now
+      case 'D': debug_level = getnum( arg, 0, 3 ); break;
       case 'f': force = true; break;
+      case 'F': recompress = true; break;
       case 'h': show_help(); return 0;
       case 'k': keep_input_files = true; break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( arg, 0, LZ_min_match_len_limit(),
-                                  LZ_max_match_len_limit() ); break;
+                  getnum( arg, LZ_min_match_len_limit(),
+                               LZ_max_match_len_limit() ); break;
       case 'o': default_output_filename = arg; break;
-      case 'n': num_workers = getnum( arg, 0, 1, max_workers ); break;
+      case 'n': num_workers = getnum( arg, 1, max_workers ); break;
       case 'q': verbosity = -1; break;
       case 's': encoder_options.dictionary_size = get_dict_size( arg );
                 break;
@@ -598,6 +595,14 @@ int main( const int argc, const char * const argv[] )
       }
     } // end process options
 
+#if defined(__OS2__)
+  _fsetmode( stdin, "b" );
+  _fsetmode( stdout, "b" );
+#endif
+
+  if( program_mode == m_test )
+    outfd = -1;
+
   if( data_size <= 0 )
     data_size = 2 * std::max( 65536, encoder_options.dictionary_size );
   else if( data_size < encoder_options.dictionary_size )
@@ -609,7 +614,6 @@ int main( const int argc, const char * const argv[] )
     if( num_online <= 0 ) num_online = 1;
     num_workers = std::min( num_online, max_workers );
     }
-  const int num_slots = num_workers * slots_per_worker;
 
   bool filenames_given = false;
   for( ; argind < parser.arguments(); ++argind )
@@ -625,8 +629,6 @@ int main( const int argc, const char * const argv[] )
   std::signal( SIGUSR1, signal_handler );
 
   Pretty_print pp( filenames );
-  if( program_mode == m_test )
-    outfd = -1;
 
   int retval = 0;
   for( unsigned int i = 0; i < filenames.size(); ++i )
@@ -662,7 +664,7 @@ int main( const int argc, const char * const argv[] )
       input_filename = filenames[i];
       const int eindex = extension_index( input_filename );
       infd = open_instream( input_filename, &in_stats, program_mode,
-                            eindex, force, to_stdout );
+                            eindex, recompress, to_stdout );
       if( infd < 0 ) { if( retval < 1 ) retval = 1; continue; }
       if( program_mode != m_test )
         {
@@ -693,11 +695,11 @@ int main( const int argc, const char * const argv[] )
     int tmp = 0;
     if( program_mode == m_compress )
       tmp = compress( data_size, encoder_options.dictionary_size,
-                      encoder_options.match_len_limit, num_workers,
-                      num_slots, infd, outfd, pp, debug_level );
+                      encoder_options.match_len_limit,
+                      num_workers, infd, outfd, pp, debug_level );
     else
-      tmp = decompress( num_workers, num_slots, infd, outfd, pp,
-                        debug_level, program_mode == m_test );
+      tmp = decompress( num_workers, infd, outfd, pp, debug_level,
+                        program_mode == m_test );
     if( tmp > retval ) retval = tmp;
     if( tmp && program_mode != m_test ) cleanup_and_fail( retval );
 
